Package: spatialFDA
Title: A Tool for Spatial Multi-sample Comparisons
Version: 1.5.1
Authors@R: c(
    person("Martin", "Emons", , "martin.emons@uzh.ch", role = c("aut", "cre"),
           comment = c(ORCID = "0009-0000-5219-5311")),
    person("Samuel", "Gunz", ,role="aut",
           comment = c(ORCID = "0000-0002-8909-0932")),
    person("Fabian", "Scheipl", , role="aut",
            comment = c(ORCID = "0000-0001-8172-3603")),
    person("Elizabeth", "Purdom", , role="aut",
            comment = c(ORCID = "0000-0001-9455-7990")),
    person("Mark D.", "Robinson", , "mark.robinson@mls.uzh.ch", role = c("aut", "fnd"),
           comment = c(ORCID = "0000-0002-3048-5518")))
URL: https://github.com/mjemons/spatialFDA
BugReports: https://github.com/mjemons/spatialFDA/issues
Description: spatialFDA is a package to calculate spatial statistics metrics. 
            The package takes a SpatialExperiment object and calculates spatial statistics metrics using the package spatstat.
            Then it compares the resulting functions across samples/conditions using functional additive models as implemented in the package refund. 
            Furthermore, it provides exploratory visualisations using functional principal component analysis, as well implemented in refund.
License: GPL (>= 3) + file LICENSE
Encoding: UTF-8
Depends: R (>= 4.3.0)
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
Imports: dplyr, ggplot2, parallel, patchwork, purrr, refund,
        SpatialExperiment, spatstat.explore, spatstat.geom,
        SummarizedExperiment, methods, stats, fda, tidyr, graphics,
        ExperimentHub, scales, S4Vectors, mgcv
biocViews: Software, Spatial, Transcriptomics
VignetteBuilder: knitr
Suggests: stringr, knitr, rmarkdown, BiocStyle, testthat (>= 3.0.0)
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/spatialFDA
git_branch: devel
git_last_commit: 0a191f4
git_last_commit_date: 2026-08-21
Repository: Bioconductor 3.24
Date/Publication: 2026-08-21
NeedsCompilation: no
Packaged: 2026-08-22 00:42:48 UTC; biocbuild
Author: Martin Emons [aut, cre] (ORCID:
    <https://orcid.org/0009-0000-5219-5311>),
  Samuel Gunz [aut] (ORCID: <https://orcid.org/0000-0002-8909-0932>),
  Fabian Scheipl [aut] (ORCID: <https://orcid.org/0000-0001-8172-3603>),
  Elizabeth Purdom [aut] (ORCID: <https://orcid.org/0000-0001-9455-7990>),
  Mark D. Robinson [aut, fnd] (ORCID:
    <https://orcid.org/0000-0002-3048-5518>)
Maintainer: Martin Emons <martin.emons@uzh.ch>
