# `spatialFDA` package sticker

* package:[URL](http://bioconductor.org/packages/spatialFDA)
* Maintainer: [Martin Emons](https://github.com/mjemons/)
* License: Creative Commons Attribution
[CC-BY](https://creativecommons.org/licenses/by/4.0/)

<img src=spatialFDA.png height="200">
