The data used to generate the outputs `gcross.csv` `lcross_single.csv` and `lcross.csv` comes from an `ExperimentHub` entry by Damond et al. (2023). The raw data to this entry can be accessed on Mendeley Data `https://data.mendeley.com/datasets/cydmwsfztj/2` published under a CC-BY-4.0 license Damond et al. (2020) with the associated publication `https://doi.org/10.1016/j.cmet.2018.11.014` by Damond et al. (2019).

link: `https://creativecommons.org/licenses/by/4.0/deed.en`
