% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.loadExample}
\alias{.loadExample}
\title{load Example dataset from Damond et al. (2019)}
\usage{
.loadExample(full = FALSE)
}
\arguments{
\item{full}{a boolean indicating whether to load the entire
Damond et al. (2019) or only a subset}
}
\value{
A SpatialExperiment object as uploaded to \code{ExperimentHub()}
}
\description{
load Example dataset from Damond et al. (2019)
}
\examples{
# retrieve the Damond et al. (2019) dataset
spe <- .loadExample()

}
