% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.speToDf}
\alias{.speToDf}
\title{Transform a SpatialExperiment into a dataframe}
\usage{
.speToDf(spe)
}
\arguments{
\item{spe}{A SpatialExperiment object subset to a single image}
}
\value{
A dataframe with the x and y coordinates from the corresponding
SpatialExperiment and the colData
}
\description{
Transform a SpatialExperiment into a dataframe
}
\examples{
# retrieve example data from Damond et al. (2019)
spe <- .loadExample()
speSub <- subset(spe, , image_number == "138")
dfSub <- .speToDf(speSub)
}
