% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotMetric.R
\name{plotCrossFOV}
\alias{plotCrossFOV}
\title{Creates a nXn plot of the cross metrics per sample}
\usage{
plotCrossFOV(
  subFov,
  theo,
  correction,
  x,
  imageId,
  ID = NULL,
  ncol = NULL,
  nrow = NULL,
  legend.position = "none",
  ...
)
}
\arguments{
\item{subFov}{a subset of the \code{dataframe} to the respective fov}

\item{theo}{logical; if the theoretical line should be plotted}

\item{correction}{the border correction to plot}

\item{x}{the x-axis variable to plot}

\item{imageId}{the ID of the image/fov}

\item{ID}{the (optional) ID for plotting combinations}

\item{ncol}{the number of columns for the facet wrap}

\item{nrow}{the number of rows for the facet wrap}

\item{legend.position}{the position of the legend of the plot}

\item{...}{Other parameters passed to \code{ggplot2} functions}
}
\value{
a ggplot object
}
\description{
Helper function for \code{plotCrossMetricPerFov}. It applies \code{plotMetricPerFov}
to all \code{n} marks defined in the variable \code{selection}. This gives an
\code{nxn} plot of all marks.
}
