% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rMaxHeuristic.R
\name{rMaxHeuristic}
\alias{rMaxHeuristic}
\title{Heuristic for the choice of rMax}
\usage{
rMaxHeuristic(spe, subsetby, marks)
}
\arguments{
\item{spe}{a \code{SpatialExperiment} object}

\item{subsetby}{the spe \code{colData} variable to subset the data by. This
variable has to be provided, even if there is only one sample.}

\item{marks}{the marks to consider e.g. cell types}
}
\value{
a ggplot histogram of the bounding radius of all the
}
\description{
Heuristic for the choice of rMax
}
\examples{
# retrieve example data from Damond et al. (2019)
spe <- .loadExample()
p <- rMaxHeuristic(spe,
subsetby = "image_number", marks = "cell_type"
)
}
