% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/funcGam.R
\name{summary.functionalGam}
\alias{summary.functionalGam}
\title{Summary for functionalGam object}
\usage{
\method{summary}{functionalGam}(object, ...)
}
\arguments{
\item{object}{a fitted \code{functionalGam}-object}

\item{...}{see \code{\link[mgcv]{summary.gam}()} for options.}
}
\value{
A list with summary information, see \code{\link[mgcv]{summary.gam}()}
}
\description{
Summary for functionalGam object
}
\author{
Martin Emons, adapted from \code{\link[refund]{summary.pffr}()} by Fabian Scheipl
}
